/*
CurrentWorkoutActivity.java created by Alex Dowlding
 */
package com.example.alex.liftcoach;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.SystemClock;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.preference.PreferenceManager;
import android.view.View;
import android.widget.Button;
import android.widget.Chronometer;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.TextView;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;

public class CurrentWorkoutActivity extends AppCompatActivity {
    private String filename;
    String workoutTitleMarker = "#####";
    String reps;
    String setsString;
    int setsCounter = 0;
    String logfileName;
    int setsInt;
    int repsInt;
    int liftIndex = 0;
    int liftCount = 0;
    ImageView imageDiagram;
    TextView weightTextView;
    Chronometer chronometer;
    Button finishWorkoutButton;
    TextView currentLiftTextView;
    TextView setsRepsTextView;
    ImageButton nextLiftButton;
    ImageButton pauseWorkoutButton;
    private long lastPause;
    boolean isChronometerRunning;
    ArrayList<String> liftList = new ArrayList<String>();

    /* #################################################################
    onCreate is the "setup" method for this activity. It sets up the on
    screen assets and creates/sets up multiple listeners.
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_current_workout);

        SetupEntities();//link all on screen assets

        filename = getString(R.string.filename);//get from resources
        logfileName = getString(R.string.logfile);//get from resources

        chronometer.start();//start the stopwatch
        isChronometerRunning = true;//necessary for enabling/disabling buttons

        finishWorkoutButton.setOnLongClickListener(new View.OnLongClickListener() {
            @Override
            public boolean onLongClick(View v) {
                //long click so it's much more difficult to accidentally hit the finish button
                chronometer.stop();
                WriteRecord();//write the record of the workout to the file
                Intent intent = new Intent(getBaseContext(), ResultsActivity.class).putExtra("chronoTime", chronometer.getText());
                startActivity(intent); //go to the activity
                return true;
            }
        });

        finishWorkoutButton.setVisibility(View.INVISIBLE);
        finishWorkoutButton.setEnabled(false);
        //Don't want to accidentally hit this button

        String workoutName = getIntent().getStringExtra("workoutName");
        //getting from the previous fragment

        RetrieveNumSets(workoutName);//get the number of sets and reps

        for (int i = 0; i < setsInt; i++)
            PopulateWorkout(liftList, workoutName);
        //Essentially, add each lift 1 * number of sets times

        String liftName = liftList.get(liftIndex);
        currentLiftTextView.setText(liftName);
        DetermineWeight(liftName, weightTextView);
        //How much weight should be lifted?

        updateTextViewLabel(setsString, reps, setsRepsTextView);
        SetImage(imageDiagram, liftName);
        //Methods that handle display info


        nextLiftButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                liftIndex++;
                String liftName = liftList.get(liftIndex);
                currentLiftTextView.setText(liftName);
                SetImage(imageDiagram, liftName);
                updateTextViewLabel(setsString, reps, setsRepsTextView);
                DetermineWeight(liftName, weightTextView);
                //Update all relevant information here when next is pressed

                if ((liftIndex + 1) >= liftList.size())
                    nextLiftButton.setEnabled(false);
                //disable once the workout is finished.
            }
        });


        pauseWorkoutButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (isChronometerRunning) //workout is not paused
                {
                    lastPause = SystemClock.elapsedRealtime();
                    chronometer.stop();//now pause
                    isChronometerRunning = false;
                    nextLiftButton.setEnabled(false);//don't let them advance while paused
                    finishWorkoutButton.setVisibility(v.VISIBLE);
                    //let them finish if they want
                    finishWorkoutButton.setEnabled(true);
                    pauseWorkoutButton.setImageResource(R.drawable.ic_play_arrow_black_24dp);
                    //change the image to play
                }

                else
                {
                    chronometer.setBase(chronometer.getBase() + SystemClock.elapsedRealtime() - lastPause);
                    //necessary - android clock keeps running even when stopped for some reason
                    //therefore necessary to manually adjust
                    chronometer.start();
                    isChronometerRunning = true;
                    nextLiftButton.setEnabled(true);
                    finishWorkoutButton.setVisibility(v.INVISIBLE);
                    finishWorkoutButton.setEnabled(false);
                    pauseWorkoutButton.setImageResource(R.drawable.ic_pause_black_24dp);
                }
            }
        });
    }

    /* ######################################################################
    PopulateWorkout gets the workoutname as a parameter and uses it to find the
    workout data.  Then it fills the ArrayList.
     */
    private void PopulateWorkout(ArrayList<String> liftList, String selectedWorkoutName) {
        try {
            String Message;
            FileInputStream fileInputStream = openFileInput(filename);
            InputStreamReader inputStreamReader = new InputStreamReader(fileInputStream);
            BufferedReader bufferedReader = new BufferedReader(inputStreamReader);

            while ((Message = bufferedReader.readLine()) != null) //while not eof
            {
                if (Message.contentEquals(selectedWorkoutName)) //found it
                {
                    Message = bufferedReader.readLine();//read sets, don't delete
                    Message = bufferedReader.readLine();//read reps, don't delete
                    Message = bufferedReader.readLine();
                    liftCount = 0;
                    while (Message != null && !Message.contentEquals(workoutTitleMarker))
                    {
                        //until it hits eof or next marker line, everything else should be valid lifts
                        liftList.add(Message);
                        liftCount++;
                        Message = bufferedReader.readLine();
                    }
                }
            }
        } catch (IOException e1) {
            e1.printStackTrace();
        }
    }


    /* ###############################################################
    In order to populate the list correctly depending on the number of sets,
    RetrieveNumSets is unfortunately necessary.  This code cannot be put in
    PopulateWorkout, because calling PopulateWorkout depends on this information
     */
    private void RetrieveNumSets(String selectedWorkoutName) {
        try {
            String Message;
            FileInputStream fileInputStream = openFileInput(filename);
            InputStreamReader inputStreamReader = new InputStreamReader(fileInputStream);
            BufferedReader bufferedReader = new BufferedReader(inputStreamReader);

            while ((Message = bufferedReader.readLine()) != null)
            {
                if (Message.contentEquals(selectedWorkoutName))
                {
                    Message = bufferedReader.readLine();//read sets
                    setsString = Message;//for the label
                    setsInt = Integer.parseInt(Message);
                    Message = bufferedReader.readLine();//read reps
                    reps = Message;//for the label
                    repsInt = Integer.parseInt(Message);
                }
            }
        } catch (IOException e1) {
            e1.printStackTrace();
        }
    }

    /* ###############################################################################
    updateTextViewLabel makes sure that the correct set number is displayed in the
    CurrentWorkoutActivity.
     */
    private void updateTextViewLabel(String sets, String reps, TextView setsRepsLabel) {
        String liftIndexString = Integer.toString(liftIndex);//retrieve index
        int currentSet = (liftIndex) % liftCount;//get current set number derived from total lifts
        if (currentSet == 0)
            setsCounter++;//one cycle of the workout complete, increase the set label
        setsRepsLabel.setText("Set " + setsCounter + " of " + sets + " (" + reps + " reps each)");
    }

    /* ##############################################################################
    SetImage gets and sets the correct lift diagram based on the current lift.  It does this
    by searching images based on the current lift name, using some shortening and parsing.
     */
    private void SetImage(ImageView diagram, String liftName) {
        String filteredLiftName = liftName.replaceAll("\\s+", "");
        //get rid of spaces
        filteredLiftName = filteredLiftName.toLowerCase(); //no uppercase
        Context context = diagram.getContext();//make sure it knows what activity to paste on
        int id = context.getResources().getIdentifier(filteredLiftName, "drawable", context.getPackageName());
        //search for the diagram matching the workout name
        diagram.setImageResource(id);
    }

    /* #######################################################################################
    WriteRecord writes an entry in a log file after the user completes the workout. It also deletes
    workouts logged older than 7 days so that the goal progress circle bar can be updated correctly.
     */
    private void WriteRecord() {
        File fileName = getBaseContext().getFileStreamPath(logfileName);
        File tempName = getBaseContext().getFileStreamPath("tempfile");
        try {
            String line;
            FileOutputStream fileTempOutputStream = getBaseContext().openFileOutput("tempfile", MODE_PRIVATE);
            //just temp so delete existing content when writing
            FileOutputStream fileOutputStream = getBaseContext().openFileOutput(logfileName, MODE_PRIVATE | MODE_APPEND);
            //Append only
            FileInputStream fileInputStream = getBaseContext().openFileInput(logfileName);
            InputStreamReader inputStreamReader = new InputStreamReader(fileInputStream);
            BufferedReader bufferedReader = new BufferedReader(inputStreamReader);

            DateFormat format = new SimpleDateFormat("MMMM d, yyyy", Locale.ENGLISH);
            //standard format
            Date cutoffDate = new Date();
            Calendar cal = GregorianCalendar.getInstance();
            cal.add(Calendar.DAY_OF_YEAR, -7);//subtract 7 days
            cutoffDate = cal.getTime();
            //Now we have date object for exactly 7 days ago
            while ((line = bufferedReader.readLine()) != null) {
                Date recordedDate = format.parse(line);
                if (!recordedDate.before(cutoffDate))
                {
                    //if newer than 7 days ago, write it to the temp file so it stays
                    fileTempOutputStream.write(line.getBytes());
                    fileTempOutputStream.write('\n');
                }
            }

            Date nowDate = new Date();
            String newDate = format.format(nowDate);
            fileTempOutputStream.write(newDate.getBytes());
            //write the current date for the workout completed.

            fileInputStream.close();
            fileOutputStream.close();
            fileTempOutputStream.close();

            fileName.delete();
            tempName.renameTo(fileName);
            //save changes by deleting old file and renaming temp file

        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (ParseException e) {
            e.printStackTrace();
        }
    }

    /* ##################################################################
    DetermineWeight calculates the amount of weight for any particlar lift,
    based on the type of lift it is as well as what set it is in the current workout.
    It does this by getting the index of the current weight for the resources array,
    then getting the type of lift by accessing the same index for the type array in
    resources.  Then based on the set info and the type, it calculates weight.
     */
    private void DetermineWeight(String liftName, TextView weightTextView) {
        String[] masterLiftArray = getResources().getStringArray(R.array.masterLiftListArray);
        String[] liftTypeArray = getResources().getStringArray(R.array.masterLiftTypeArray);
        //get copies of both arrays
        int index;
        for (index = 0; index < masterLiftArray.length; index++)
        {
            //search for the lift
            if (masterLiftArray[index].contentEquals(liftName))
                break;
        }

        SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(getBaseContext());
        //get copy of preferences for max weights

        double repsMultiplier = (double) repsInt / 8;
        //used to modify weight
        double weight = 100; //WILL BE CHANGED LATER
        //still must be initialized here.

        if (liftTypeArray[index].contentEquals("Chest")) //chest exercise
        {
            String maxString = prefs.getString("benchMax", "135");
            //get the max weight for bench
            double max = Double.parseDouble(maxString);

            //if repsMultiplier less than .8, increase weight
            //less reps equals more weight to be effective
            if (((double) setsCounter / (double) setsInt) > .75)
            {
                weight = max * 0.8;
                if (repsMultiplier < .8)
                    weight += (weight * .1);
                else if (repsMultiplier > 1.2)
                    weight -= (weight * .1);
            }
            else if (((double) setsCounter / (double) setsInt) > .50)
            {
                weight = max * 0.7;
                if (repsMultiplier < .8)
                    weight += (weight * .1);
                else if (repsMultiplier > 1.2)
                    weight -= (weight * .1);
            }
            else
            {
                weight = max * 0.6;
                if (repsMultiplier < .8)
                    weight += (weight * .1);
                else if (repsMultiplier > 1.2)
                    weight -= (weight * .1);
            }
        }

        if (liftTypeArray[index].contentEquals("Shoulders")) //shoulder exercise
        {
            String maxString = prefs.getString("benchMax", "135");
            //can still take from bench
            double max = Double.parseDouble(maxString);

            //if repsMultiplier less than .8, increase weight
            //less reps equals more weight to be effective
            if (((double) setsCounter / (double) setsInt) > .75)
            {
                weight = max * 0.7;
                if (repsMultiplier < .8)
                    weight += (weight * .1);
                else if (repsMultiplier > 1.2)
                    weight -= (weight * .1);
            }
            else if (((double) setsCounter / (double) setsInt) > .50)
            {
                weight = max * 0.6;
                if (repsMultiplier < .8)
                    weight += (weight * .1);
                else if (repsMultiplier > 1.2)
                    weight -= (weight * .1);
            }
            else
            {
                weight = max * 0.5;
                if (repsMultiplier < .8)
                    weight += (weight * .1);
                else if (repsMultiplier > 1.2)
                    weight -= (weight * .1);
            }
        }

        if (liftTypeArray[index].contentEquals("Arms")) //arm lift
        {
            String maxString = prefs.getString("benchMax", "135");
            //can still use bench, but just lighter weights than shoulders or chest
            double max = Double.parseDouble(maxString);

            //if repsMultiplier less than .8, increase weight
            //less reps equals more weight to be effective
            if (((double) setsCounter / (double) setsInt) > .75)
            {
                weight = max * 0.65;
                if (repsMultiplier < .8)
                    weight += (weight * .1);
                else if (repsMultiplier > 1.2)
                    weight -= (weight * .1);
            }
            else if (((double) setsCounter / (double) setsInt) > .50)
            {
                weight = max * 0.55;
                if (repsMultiplier < .8)
                    weight += (weight * .1);
                else if (repsMultiplier > 1.2)
                    weight -= (weight * .1);
            }
            else
            {
                weight = max * 0.45;
                if (repsMultiplier < .8)
                    weight += (weight * .1);
                else if (repsMultiplier > 1.2)
                    weight -= (weight * .1);
            }
        }

        if (liftTypeArray[index].contentEquals("Legs")) //legs workout
        {
            String maxString = prefs.getString("squatMax", "135");
            //use squat max
            double max = Double.parseDouble(maxString);

            //if repsMultiplier less than .8, increase weight
            //less reps equals more weight to be effective

            if (((double) setsCounter / (double) setsInt) > .75)
            {
                weight = max * 0.8;
                if (repsMultiplier < .8)
                    weight += (weight * .1);
                else if (repsMultiplier > 1.2)
                    weight -= (weight * .1);
            }
            else if (((double) setsCounter / (double) setsInt) > .50)
            {
                weight = max * 0.7;
                if (repsMultiplier < .8)
                    weight += (weight * .1);
                else if (repsMultiplier > 1.2)
                    weight -= (weight * .1);
            }
            else
            {
                weight = max * 0.6;
                if (repsMultiplier < .8)
                    weight += (weight * .1);
                else if (repsMultiplier > 1.2)
                    weight -= (weight * .1);
            }
        }

        long longWeight = 5 * (Math.round(weight / 5));
        //free weights usually only in increments of 5 or 10
        weightTextView.setText((int) longWeight + " lbs");
        //update label
    }

    /* ##################################################################
    SetupEntities links the on screen widgets to code so that they can
    be manipulated.
     */
    private void SetupEntities()
    {
        imageDiagram = (ImageView) findViewById(R.id.liftDiagramImageView);
        weightTextView = (TextView) findViewById(R.id.weightTextView);
        finishWorkoutButton = (Button) findViewById(R.id.finishWorkoutButton);
        chronometer = (Chronometer) findViewById(R.id.workoutChronometer);
        currentLiftTextView = (TextView) findViewById(R.id.currentLiftTextView);
        nextLiftButton = (ImageButton) findViewById(R.id.nextLiftButton);
        setsRepsTextView = (TextView) findViewById(R.id.setsRepsTextView);
        pauseWorkoutButton = (ImageButton) findViewById(R.id.pauseWorkoutButton);
    }
}
