/*
chooseWorkoutActivity created by Alex Dowlding
Acts as the screen on which the user selects which workout
they will use.
 */
package com.example.alex.liftcoach;

import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;


public class chooseWorkoutActivity extends AppCompatActivity {

    String workoutTitleMarker = "#####";
    Button startButton;
    private ArrayAdapter<String> workoutListAdapter;
    private ArrayAdapter<String> workoutPreviewAdapter;
    private String filename;
    private String sentWorkoutName;
    ArrayList<String> workoutList = new ArrayList<String>();
    ArrayList<String> previewList = new ArrayList<String>();
    ListView workouts;
    ListView workoutsPreview;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_choose_workout);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        //above is auto generated code that sets up back button

        filename = getString(R.string.filename);//get from resource file, must be here
        SetupEntities(); //set up on screen entities
        ReadFile(); //read in the workouts

        workoutListAdapter = new ArrayAdapter<String>(getBaseContext(),android.R.layout.simple_list_item_single_choice, workoutList);
        //load adapter from the workoutList array
        workouts.setAdapter(workoutListAdapter);//link the adapter

        workoutPreviewAdapter = new ArrayAdapter<String>(getBaseContext(), android.R.layout.simple_list_item_1, previewList);
        //load adapter from previewList array
        workoutsPreview.setAdapter(workoutPreviewAdapter);//link the adapter

        //create and set a listener for the start button
        startButton.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent intent = new Intent(getBaseContext(), CurrentWorkoutActivity.class).putExtra("workoutName", sentWorkoutName);
                startActivity(intent);//go to the currentWorkout activity
            }
        });
        startButton.setEnabled(false);//not enabled until user selects a workout

        //create and set a listener for the workout list
        workouts.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                sentWorkoutName =(String) (workouts.getItemAtPosition(position));
                //retrieve name of selected workout
                startButton.setEnabled(true); //allow user to start
                ChangePreviewList(previewList, sentWorkoutName);
                //loads preview list for selected workout in real time.
            }
        });
    }

    /* ##################################################################################
    ReadFile searches the file for workoutnames and loads the names into the workoutlist.
    Then the user can select one.
     */
    private void ReadFile()
    {
        try {
            String Message;
            FileInputStream fileInputStream = openFileInput(filename);
            InputStreamReader inputStreamReader = new InputStreamReader(fileInputStream);
            BufferedReader bufferedReader = new BufferedReader(inputStreamReader);
            //StringBuffer stringBuffer = new StringBuffer();
            int index = 0;

            while ((Message = bufferedReader.readLine()) != null)
            {
                if(Message.contentEquals(workoutTitleMarker))//we found a workout
                {
                    Message = bufferedReader.readLine();
                    workoutList.add(index, Message);//add the found workout name to the list
                    index++;
                }
            }
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /* #################################################################################
    ChangePreviewList changes the previewlist based off of which workout is selected at a given time.
    It finds the name of the selected workout in file, and adds subsequent lifts to the list.
     */
    private void ChangePreviewList(List<String> previewList, String selectedWorkoutName)
    {
        try {
            previewList.clear();//make sure empty so no duplicates on resume
            String Message;
            FileInputStream fileInputStream = openFileInput(filename);
            InputStreamReader inputStreamReader = new InputStreamReader(fileInputStream);
            BufferedReader bufferedReader = new BufferedReader(inputStreamReader);
            //StringBuffer stringBuffer = new StringBuffer();
            while ((Message = bufferedReader.readLine()) != null)//while not end of file
            {
                if (Message.contentEquals(selectedWorkoutName))//if selectedworkout found
                {
                    Message = bufferedReader.readLine();//sets, unused but don't delete
                    Message = bufferedReader.readLine();//reps, unused but don't delete
                    int index = 0;
                    Message = bufferedReader.readLine();
                    while(Message != null && !Message.contentEquals(workoutTitleMarker))
                    {
                        previewList.add(index, Message);//add all lifts in that workout to preview list
                        Message = bufferedReader.readLine();
                    }
                }
            }
            workoutPreviewAdapter.notifyDataSetChanged();//notifies to redraw
        } catch (IOException e1) {
            e1.printStackTrace();
        }
    }

    /* ########################################################################
    SetupEntities links variables to the on screen entities so that they can be
    manipulated programatically.
     */
    private void SetupEntities()
    {
        workouts = (ListView) findViewById(R.id.chooseWorkoutListView);
        workoutsPreview = (ListView) findViewById(R.id.chooseWorkoutPreviewListView);
        startButton = (Button) findViewById(R.id.startButton);
    }
}
